import 'dart:collection';
import 'dart:io';
import 'package:buzz_booster_example/ServerMutator.dart';
import 'package:shared_preferences/shared_preferences.dart';

const String defaultAndroidAppKey = "307117684877774";
const String defaultiosAppKey = "279753136766115";
const String androidAppKeyKey = "androidAppKey";
const String androidAppKeyListKey = "androidAppKeyList";
const String iosAppKeyKey = "iosAppKey";
const String iosAppKeyListKey = "iosAppKeyList";
const String userIdKey = "userIdKey";
const String serverEnvironmentKey = "getServerEnvironmentKey";

class Repository {
  Future<SharedPreferences> _prefs = SharedPreferences.getInstance();

  Future<String> getAppKey() async {
    if (Platform.isAndroid) {
      return getAndroidAppKey();
    } else {
      return getiOSAppKey();
    }
  }

  Future<Environment> getServerEnvironment() async {
    final SharedPreferences prefs = await _prefs;
    final string = prefs.getString(serverEnvironmentKey);
    if (string == null) {
      return Future.value(Environment.Production);
    } else {
      final env = Environment.values.firstWhere((element) => element.toString() == string);
      return Future.value(env);
    }
  }

  Future<void> saveServerEnvironment(Environment environment) async {
    final SharedPreferences prefs = await _prefs;
    prefs.setString(serverEnvironmentKey, environment.toString());
  }

  Future<String> getAndroidAppKey() async {
    final SharedPreferences prefs = await _prefs;
    return prefs.getString(androidAppKeyKey) ?? defaultAndroidAppKey;
  }

  Future<String> getiOSAppKey() async {
    final SharedPreferences prefs = await _prefs;
    return prefs.getString(iosAppKeyKey) ?? defaultiosAppKey;
  }

  Future<String> getUserId() async {
    final SharedPreferences prefs = await _prefs;
    return prefs.getString(userIdKey) ?? "Damon";
  }

  Future<void> saveAppKey(String appKey, String userId) async {
    final SharedPreferences prefs = await _prefs;
    if (Platform.isAndroid) {
      prefs.setString(androidAppKeyKey, appKey);
    } else {
      prefs.setString(iosAppKeyKey, appKey);
    }
    prefs.setString(userIdKey, userId);
  }

  Future<void> saveAppKeyToList(String appKey) async {
    final SharedPreferences prefs = await _prefs;
    if (Platform.isAndroid) {
      List<String> appKeys = prefs.getStringList(androidAppKeyListKey) ?? [];
      appKeys.add(appKey);
      prefs.setStringList(androidAppKeyListKey, appKeys);
    } else {
      List<String> appKeys = prefs.getStringList(iosAppKeyListKey) ?? [];
      appKeys.add(appKey);
      prefs.setStringList(iosAppKeyListKey, appKeys);
    }
  }
  Future<String> nextAppKeyInAppKeys() async {
    final SharedPreferences prefs = await _prefs;
    if (Platform.isAndroid) {
      List<String> appKeys = prefs.getStringList(androidAppKeyListKey) ?? [defaultAndroidAppKey];
      Queue<String> queue = Queue.from(appKeys);
      if (queue.length == 1) {
        return Future.value(queue.first);    
      } else {
        String appKey = queue.removeFirst();
        queue.add(appKey);
        prefs.setStringList(androidAppKeyListKey, queue.toList());
        return Future.value(appKey);
      }
    } else {
      List<String> appKeys = prefs.getStringList(iosAppKeyListKey) ?? [defaultiosAppKey];
      Queue<String> queue = Queue.from(appKeys);
      if (queue.length == 1) {
        return Future.value(queue.first);    
      } else {
        String appKey = queue.removeFirst();
        queue.add(appKey);
        prefs.setStringList(iosAppKeyListKey, queue.toList());
        return Future.value(appKey);
      }
    }
  }
}
